//
//  AZBpmFinder.m
//  AubioTest
//
//  Created by Paolo Rossignoli on 14/01/14.
//  Copyright (c) 2014 Paolo Rossignoli. All rights reserved.
//

#import "AZBpmFinder.h"

#define AUBIO_UNSTABLE 1
#include <aubio/aubio.h>


#define EXTENSION       @"mp3"
#define tempo_method    "default"

@interface AZBpmFinder ()

@property (nonatomic, readwrite) float bpm;

@end


@implementation AZBpmFinder


aubio_tempo_t * tempo;
fvec_t * tempo_out;
smpl_t is_beat = 0;

#pragma mark AUBIO

- (id) init {
    self = [super init];
    if (self) {
        
    }
    return self;
}

-(char*)getCharFromFilename:(NSURL *)pFilename {
    
    NSString *path = [NSString stringWithFormat:@"%@",[pFilename relativePath]];
    char *temp = (char *)[path UTF8String];
    return temp;
}


- (void) getBpmFromSong:(NSString *)songTitle {
    
    dispatch_async(dispatch_get_global_queue( DISPATCH_QUEUE_PRIORITY_LOW, 0), ^(void){
        
        NSData*data = [self getDataFormSongTitle:songTitle];
        NSURL *pFilename = [[NSBundle mainBundle]
                            URLForResource:songTitle withExtension:EXTENSION];
        
        NSLog(@"songTitle %@ - pFilename %@",songTitle, pFilename );
        
        NSError *error = nil;
        AVAudioPlayer *player =[[AVAudioPlayer alloc] initWithData:data error:&error];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            
            float duration = player.duration;
            uint_t samplerate =  (uint_t)[[player.settings valueForKeyPath:@"AVSampleRateKey"] longValue];
            uint_t win_size = 1024; // window size
            uint_t n_frames = 0, read = 0;
            uint_t hop_size = win_size / 4;
            
            char_t *source_path = [self getCharFromFilename:pFilename];
            aubio_source_t * source = new_aubio_source(source_path, samplerate, hop_size);
            
            //if (!source) { err = 1; goto beach; }
            if (samplerate == 0 ) samplerate = aubio_source_get_samplerate(source);
            
            
            fvec_t * in = new_fvec (hop_size); // input audio buffer
            fvec_t * out = new_fvec (2); // output position
            
            // create tempo object
            //aubio_tempo_t *
            
            tempo = new_aubio_tempo("default", win_size, hop_size, samplerate);
            
            
            do {
                // put some fresh data in input vector
                aubio_source_do(source, in, &read);
                // execute tempo
                aubio_tempo_do(tempo,in,out);
                
                // do something with the beats
                if (out->data[0] != 0) {
                    /*
                     NSLog(@"beat at %.3fms, %.3fs, frame %d, %.2fbpm with confidence %.2f\n",
                     aubio_tempo_get_last_ms(o), aubio_tempo_get_last_s(o),
                     aubio_tempo_get_last(o), aubio_tempo_get_bpm(o), aubio_tempo_get_confidence(o));
                     */
                    
                    //INTERFACE DON'T RESPOND IN REAL TIME
                    if( self.delegate != nil && [ self.delegate respondsToSelector: @selector( totalAnalizedSong: ) ] )
                    {
                        float progress = (aubio_tempo_get_last_s(tempo)/duration);
                        [self.delegate totalAnalizedSong:progress];
                    }
                    
                }
                
                n_frames += read;
                
            } while ( read == hop_size );
            NSLog(@"read %.2fs, %d frames at %dHz (%d blocks) from %s\n",
                  n_frames * 1. / samplerate,
                  n_frames,
                  samplerate,
                  n_frames / hop_size,
                  source_path);
            
            
            self.bpm = aubio_tempo_get_bpm(tempo);
            
            if( self.delegate != nil && [ self.delegate respondsToSelector: @selector( beatsPerMinutes: ) ] )
            {
                [self.delegate beatsPerMinutes:_bpm];
                NSLog(@"**** self.bpm %f",_bpm);
            }
            
            // clean up memory
            del_aubio_tempo(tempo);
            del_fvec(in);
            del_fvec(out);
            del_aubio_source(source);
            
        });
    });
}


- (NSData*) getDataFormSongTitle:(NSString*)songTitle {
    NSString *path = [[NSBundle mainBundle] pathForResource:songTitle ofType:EXTENSION];
    NSData *data = [NSData dataWithContentsOfFile:path];
    return data;
}


#pragma mark -
#pragma mark Aubio delegate




@end
